@@ -43,6 +43,7 @@ windows_connector_list = [
     'os_brick.initiator.windows.base.BaseWindowsConnector',
     'os_brick.initiator.windows.iscsi.WindowsISCSIConnector',
     'os_brick.initiator.windows.fibre_channel.WindowsFCConnector',
+    'os_brick.initiator.windows.rbd.WindowsRBDConnector',
     'os_brick.initiator.windows.smbfs.WindowsSMBFSConnector'
 ]
 
@@ -54,18 +55,13 @@ unix_connector_list = [
      'FibreChannelConnectorS390X'),
     ('os_brick.initiator.connectors.fibre_channel_ppc64.'
      'FibreChannelConnectorPPC64'),
-    'os_brick.initiator.connectors.aoe.AoEConnector',
     'os_brick.initiator.connectors.remotefs.RemoteFsConnector',
     'os_brick.initiator.connectors.rbd.RBDConnector',
     'os_brick.initiator.connectors.local.LocalConnector',
     'os_brick.initiator.connectors.gpfs.GPFSConnector',
-    'os_brick.initiator.connectors.drbd.DRBDConnector',
     'os_brick.initiator.connectors.huawei.HuaweiStorHyperConnector',
-    'os_brick.initiator.connectors.hgst.HGSTConnector',
     'os_brick.initiator.connectors.scaleio.ScaleIOConnector',
-    'os_brick.initiator.connectors.disco.DISCOConnector',
     'os_brick.initiator.connectors.vmware.VmdkConnector',
-    'os_brick.initiator.connectors.vrtshyperscale.HyperScaleConnector',
     'os_brick.initiator.connectors.storpool.StorPoolConnector',
     'os_brick.initiator.connectors.nvmeof.NVMeOFConnector',
 ]
@@ -80,11 +76,6 @@ def _get_connector_list():
 
 # Mappings used to determine who to construct in the factory
 _connector_mapping_linux = {
-    initiator.AOE:
-        'os_brick.initiator.connectors.aoe.AoEConnector',
-    initiator.DRBD:
-        'os_brick.initiator.connectors.drbd.DRBDConnector',
-
     initiator.GLUSTERFS:
         'os_brick.initiator.connectors.remotefs.RemoteFsConnector',
     initiator.NFS:
@@ -107,22 +98,14 @@ _connector_mapping_linux = {
         'os_brick.initiator.connectors.local.LocalConnector',
     initiator.HUAWEISDSHYPERVISOR:
         'os_brick.initiator.connectors.huawei.HuaweiStorHyperConnector',
-    initiator.HGST:
-        'os_brick.initiator.connectors.hgst.HGSTConnector',
     initiator.RBD:
         'os_brick.initiator.connectors.rbd.RBDConnector',
     initiator.SCALEIO:
         'os_brick.initiator.connectors.scaleio.ScaleIOConnector',
-    initiator.DISCO:
-        'os_brick.initiator.connectors.disco.DISCOConnector',
-    initiator.SHEEPDOG:
-        'os_brick.initiator.connectors.sheepdog.SheepdogConnector',
     initiator.VMDK:
         'os_brick.initiator.connectors.vmware.VmdkConnector',
     initiator.GPFS:
         'os_brick.initiator.connectors.gpfs.GPFSConnector',
-    initiator.VERITAS_HYPERSCALE:
-        'os_brick.initiator.connectors.vrtshyperscale.HyperScaleConnector',
     initiator.STORPOOL:
         'os_brick.initiator.connectors.storpool.StorPoolConnector',
     # Leave this in for backwards compatibility
@@ -138,8 +121,6 @@ _connector_mapping_linux_s390x = {
     initiator.FIBRE_CHANNEL:
         'os_brick.initiator.connectors.fibre_channel_s390x.'
         'FibreChannelConnectorS390X',
-    initiator.DRBD:
-        'os_brick.initiator.connectors.drbd.DRBDConnector',
     initiator.NFS:
         'os_brick.initiator.connectors.remotefs.RemoteFsConnector',
     initiator.ISCSI:
@@ -157,8 +138,6 @@ _connector_mapping_linux_ppc64 = {
     initiator.FIBRE_CHANNEL:
         ('os_brick.initiator.connectors.fibre_channel_ppc64.'
          'FibreChannelConnectorPPC64'),
-    initiator.DRBD:
-        'os_brick.initiator.connectors.drbd.DRBDConnector',
     initiator.NFS:
         'os_brick.initiator.connectors.remotefs.RemoteFsConnector',
     initiator.ISCSI:
@@ -171,12 +150,8 @@ _connector_mapping_linux_ppc64 = {
         'os_brick.initiator.connectors.gpfs.GPFSConnector',
     initiator.VZSTORAGE:
         'os_brick.initiator.connectors.remotefs.RemoteFsConnector',
-    initiator.VERITAS_HYPERSCALE:
-        'os_brick.initiator.connectors.vrtshyperscale.HyperScaleConnector',
     initiator.ISER:
         'os_brick.initiator.connectors.iscsi.ISCSIConnector',
-    initiator.SHEEPDOG:
-        'os_brick.initiator.connectors.sheepdog.SheepdogConnector',
 }
 
 # Mapping for the windows connectors
@@ -185,6 +160,8 @@ _connector_mapping_windows = {
         'os_brick.initiator.windows.iscsi.WindowsISCSIConnector',
     initiator.FIBRE_CHANNEL:
         'os_brick.initiator.windows.fibre_channel.WindowsFCConnector',
+    initiator.RBD:
+        'os_brick.initiator.windows.rbd.WindowsRBDConnector',
     initiator.SMBFS:
         'os_brick.initiator.windows.smbfs.WindowsSMBFSConnector',
 }
@@ -233,8 +210,8 @@ def get_connector_properties(root_helper, my_ip, multipath, enforce_multipath,
     :param multipath: Enable multipath?
     :type multipath: bool
     :param enforce_multipath: Should we enforce that the multipath daemon is
-                              running?  If the daemon isn't running then the
-                              return dict will have multipath as False.
+                              running?  If the daemon isn't running then raise
+                              ProcessExecutionError to the caller.
     :type enforce_multipath: bool
     :param host: hostname.
     :param execute: execute helper.
